"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongodb_1 = require("mongodb");
const DatabaseDriver_1 = require("./DatabaseDriver");
const MongoConnection_1 = require("../connections/MongoConnection");
const utils_1 = require("../utils");
const MongoPlatform_1 = require("../platforms/MongoPlatform");
class MongoDriver extends DatabaseDriver_1.DatabaseDriver {
    constructor(config) {
        super(config, ['mongodb']);
        this.connection = new MongoConnection_1.MongoConnection(this.config);
        this.platform = new MongoPlatform_1.MongoPlatform();
    }
    async find(entityName, where, options, ctx) {
        where = this.renameFields(entityName, where);
        const res = await this.getConnection('read').find(entityName, where, options.orderBy, options.limit, options.offset, options.fields, ctx);
        return res.map((r) => this.mapResult(r, this.metadata.get(entityName)));
    }
    async findOne(entityName, where, options = { populate: [], orderBy: {} }, ctx) {
        if (utils_1.Utils.isPrimaryKey(where)) {
            where = { _id: new mongodb_1.ObjectId(where) };
        }
        where = this.renameFields(entityName, where);
        const res = await this.getConnection('read').find(entityName, where, options.orderBy, 1, undefined, options.fields, ctx);
        return this.mapResult(res[0], this.metadata.get(entityName));
    }
    async count(entityName, where, ctx) {
        where = this.renameFields(entityName, where);
        return this.getConnection('read').countDocuments(entityName, where, ctx);
    }
    async nativeInsert(entityName, data, ctx) {
        data = this.renameFields(entityName, data);
        return this.getConnection('write').insertOne(entityName, data, ctx);
    }
    async nativeUpdate(entityName, where, data, ctx) {
        if (utils_1.Utils.isPrimaryKey(where)) {
            where = { _id: new mongodb_1.ObjectId(where) };
        }
        where = this.renameFields(entityName, where);
        data = this.renameFields(entityName, data);
        return this.getConnection('write').updateMany(entityName, where, data, ctx);
    }
    async nativeDelete(entityName, where, ctx) {
        if (utils_1.Utils.isPrimaryKey(where)) {
            where = { _id: new mongodb_1.ObjectId(where) };
        }
        where = this.renameFields(entityName, where);
        return this.getConnection('write').deleteMany(entityName, where, ctx);
    }
    async aggregate(entityName, pipeline, ctx) {
        return this.getConnection('read').aggregate(entityName, pipeline, ctx);
    }
    async createCollections() {
        const promises = Object.values(this.metadata.getAll())
            .map(meta => this.getConnection('write').createCollection(meta.collection));
        await Promise.all(promises);
    }
    async dropCollections() {
        const db = this.getConnection('write').getDb();
        const collections = await db.listCollections().toArray();
        const existing = collections.map(c => c.name);
        const promises = Object.values(this.metadata.getAll())
            .filter(meta => existing.includes(meta.collection))
            .map(meta => this.getConnection('write').dropCollection(meta.collection));
        await Promise.all(promises);
    }
    async ensureIndexes() {
        await this.createCollections();
        const promises = [];
        const createIndexes = (meta, type) => {
            meta[type].forEach(index => {
                const properties = utils_1.Utils.flatten(utils_1.Utils.asArray(index.properties).map(prop => meta.properties[prop].fieldNames));
                promises.push(this.getConnection('write').getCollection(meta.name).createIndex(properties, Object.assign({ name: index.name, unique: type === 'uniques' }, (index.options || {}))));
            });
        };
        for (const meta of Object.values(this.metadata.getAll())) {
            createIndexes(meta, 'indexes');
            createIndexes(meta, 'uniques');
        }
        await Promise.all(promises);
    }
    renameFields(entityName, data) {
        data = Object.assign({}, data); // copy first
        utils_1.Utils.renameKey(data, 'id', '_id');
        const meta = this.metadata.get(entityName, false, false);
        Object.keys(data).forEach(k => {
            if (meta && meta.properties[k]) {
                const prop = meta.properties[k];
                if (prop.fieldNames) {
                    utils_1.Utils.renameKey(data, k, prop.fieldNames[0]);
                }
            }
        });
        return data;
    }
}
exports.MongoDriver = MongoDriver;
